require 'default:combat'
---------------------------------------------------------
-- OVERRIDE INFO
---------------------------------------------------------
-- we want to check for our wands and staves for damage
-- calculations. This is a direct copy paste of citadels
-- core ApplyDamageToTarget function.
---------------------------------------------------------
function ApplyDamageToTarget(damageInfo)
	Verbose("Combat", "ApplyDamageToTarget", damageInfo)
	if damageInfo == nil then return end
	damageInfo.Attacker = damageInfo.Attacker or this
	if ( damageInfo.Attacker ~= damageInfo.Victim and not ValidCombatTarget(damageInfo.Attacker, damageInfo.Victim)) then return end
	if ( damageInfo.Source ) then
		damageInfo.WeaponType = Weapon.GetType(damageInfo.Source)
		damageInfo.WeaponClass = Weapon.GetClass(damageInfo.WeaponType)
		damageInfo.WeaponSkill = Weapon.GetSkill(damageInfo.WeaponType)
		if not( damageInfo.Type ) then
			damageInfo.Type = Weapon.GetDamageType(damageInfo.WeaponType)
		end
	end
	if not( damageInfo.Type ) then damageInfo.Type = "TrueDamage" end
	local typeData = CombatDamageType[damageInfo.Type]
	-- not a real combat damage type, stop here.
	if not( typeData ) then
		LuaDebugCallStack("[ApplyDamageToTarget] invalid type: "..tostring(damageInfo.Type))
		return
	end
	-- pass in some variables
	damageInfo.VictimIsPlayer = IsPlayerCharacter(damageInfo.Victim)
	damageInfo.Owner = _MyOwner
	-- alter the damageInfo table
	damageInfo = CalculateDamageInfo(damageInfo)
	if( damageInfo.WeaponSkill == "SlashingSkill" or damageInfo.WeaponSkill == "BashingSkill" or damageInfo.WeaponSkill == "LancingSkill" or damageInfo.WeaponSkill == "PiercingSkill" or damageInfo.WeaponSkill == "ArcherySkill" ) then
		RequestMobileMod( this, damageInfo.Attacker, {"ArcherySkillDamageTimes", "PiercingSkillDamageTimes", "LancingSkillDamageTimes", "SlashingSkillDamageTimes", "BashingSkillDamageTimes"},
		function(MobileMod) 
			local damageMod = GetMobileMod(MobileMod[damageInfo.WeaponSkill.."DamageTimes"], 1)
			--DebugMessage( "damageMod", damageMod )
			damageInfo.Damage = (damageInfo.Damage * damageMod)
			damageInfo.Victim:SendMessage("DamageInflicted",damageInfo.Attacker,damageInfo.Damage,damageInfo.Type,false,damageInfo.Blocked > 0,damageInfo.IsReflected,damageInfo.Source)
		end)
	elseif( damageInfo.WeaponClass == "AirMagicStaff" or damageInfo.WeaponClass == "EarthMagicStaff" or damageInfo.WeaponClass == "FireMagicStaff" or damageInfo.WeaponClass == "WaterMagicStaff" ) then
		damageInfo.Damage = EquipmentStats.BaseWeaponStats[damageInfo.WeaponType].Attack or 10
		damageInfo.Victim:SendMessage("DamageInflicted",damageInfo.Attacker,damageInfo.Damage,damageInfo.Type,false,damageInfo.Blocked > 0,damageInfo.IsReflected,damageInfo.Source)
	elseif( damageInfo.WeaponClass == "AirMagicWand" or damageInfo.WeaponClass == "EarthMagicWand" or damageInfo.WeaponClass == "FireMagicWand" or damageInfo.WeaponClass == "WaterMagicWand" ) then
		damageInfo.Damage = EquipmentStats.BaseWeaponStats[damageInfo.WeaponType].Attack or 5
		damageInfo.Victim:SendMessage("DamageInflicted",damageInfo.Attacker,damageInfo.Damage,damageInfo.Type,false,damageInfo.Blocked > 0,damageInfo.IsReflected,damageInfo.Source)				
	else
		damageInfo.Victim:SendMessage("DamageInflicted",damageInfo.Attacker,damageInfo.Damage,damageInfo.Type,false,damageInfo.Blocked > 0,damageInfo.IsReflected,damageInfo.Source)
	end
end

---------------------------------------------------------
-- OVERRIDE INFO
---------------------------------------------------------
-- This is a direct copy paste of citadels core CheckHitSuccess
-- function. We are adding in skill checks for two-weapon skill
-- and tactics skill checks.
---------------------------------------------------------
function CheckHitSuccess(victim, hand)
	Verbose("Combat", "CheckHitSuccess", victim, hand)
	local isPlayer = IsPlayerCharacter(this)
	local accuracy = this:GetStatValue("Accuracy")
	local evasion = victim:GetStatValue("Evasion")
	local hitChance = (( accuracy ) / ( ( evasion) * 2 ) * 100) + ( this:GetStatValue("HitChance") - victim:GetStatValue("DefenseChance") )
	local hitSuccess = false
	-- if player or tamed pet
	if ( isPlayer or _MyOwner ~= nil ) then
		-- calc hit chance
        --To tweak hitchance vs monsters
        local victimIsPlayer = victim:IsPlayer()
		if not ( victimIsPlayer ) then
			hitSuccess = Success(hitChance/87.5)
		else
		--To tweak hitchance vs players
			if( _MyOwner ~= nil ) then
				hitChance = ServerSettings.Pets.Combat.PVPHitChance
			end
			--DebugMessage("Hitting Player", tostring(isPlayer), tostring(_MyOwner), tostring(hitChance))
			hitSuccess = Success(hitChance/100)	
		end
		-- Ensure only gain off valid targets
		if ( ValidCombatGainTarget(victim, _MyOwner or this) ) then
			local victimWeaponSkillLevel = GetSkillLevel(victim, victimIsPlayer and GetPrimaryWeaponSkill(victim) or "BrawlingSkill")
			
			if ( isPlayer ) then
				-- check weapon skill on hit/miss
				local weaponClassInfo = EquipmentStats.BaseWeaponClass[_Weapon[hand].Class]
				local weaponSkill = GetPrimaryWeaponSkill(this)
				if ( weaponClassInfo and weaponClassInfo.WeaponSkill and not(weaponClassInfo.WeaponSkillGainsDisabled) ) then
					CheckSkill(this, weaponSkill, victimWeaponSkillLevel)
					CheckSkill(this, "MeleeSkill", victimWeaponSkillLevel)
				end

				-- check if this is a two handed weapon
				if(weaponClassInfo.TwoHandedWeapon == true) then
					CheckSkill(this, "CombatFocusTwoHandedSkill", victimWeaponSkillLevel)
				end
			else
				-- owners can only gain if the pet didn't miss and they are within range of the pet.
				if ( hitSuccess and _MyOwner:IsValid() and this:DistanceFrom(_MyOwner) <= ServerSettings.Pets.Command.Distance ) then
					-- do a gain on owner's beastmastery
					--CheckSkill(_MyOwner, "BeastmasterySkill", victimWeaponSkillLevel)
				end
			end
		end
	else
		-- mobs cannot have a hit chance lower than 50%.
		hitSuccess = Success(math.max(hitChance/100, 0.75))
	end
	return hitSuccess
end