require 'custom_spellbook_uioptions'

--- adds the user action to the custom spellbook dynamicw window
-- @param dynamiceWindow - (dynamicwindow object) the dynamic window to put the user action on
-- @param x - (number) the x position of the user action
-- @param y - (number) the y position of the user action
-- @param spell - (spell key)(string) the spell table key
function AddUserAction(dynamicWindow,x,y,spell)
	dynamicWindow:AddUserAction(x,y,GetSpellUserAction(spell),48,48)
end

--- builds the spell user action table
-- @param spell - (table key)(string) the name of the spell being used
function GetSpellUserAction(spell)
	local spellTable = SpellData.AllSpells[spell]
	local actionTable = 
	{
		ID = spell,
		ActionType = "Spell",
		DisplayName = spellTable.DisplayName,
		Tooltip = spellTable.SpellTooltipString,
		Icon = spellTable.Icon,
		IconText = spellTable.DisplayName,
		Enabled = true,
		ServerCommand = "sp "..spell,
		Requirements = {}		
	}
	return actionTable
end	

--- gets a list of the sorted spells for this book based on school
--TODO: Make it so we only pull the spells we can actually cast at our level.
function GetSpellListSorted()
	local spellList = {}
	--sort the spells by string display name instead of table key string
	for spellKey,spellValue in pairs(SpellData.AllSpells) do
		if(spellValue.Skill == this:GetObjVar("School")) then
			--TODO: SHOULD WE ADD THIS SPELL?
			table.insert(spellList,{Name=SpellData.AllSpells[spellKey].SpellDisplayName,Key=spellKey})
		end
	end
 	-- sort this shit
 	table.sort(spellList, function(a,b) 
		return a.Name < b.Name 
	end)
 	return spellList
end

--- handles the module attached event
function HandleModuleAttached()
	AddUseCase(this,"Open Book",true)
end

--- handles the use object event
-- @param user - (game object)(player)
-- @param usedType - (string) the addusecase idtentifer or default action string
function HandleUseObject(user,usedType)
	if(usedType == "Open Book") then
		ShowSpellBook(user)
	end
end

--- handles when the user drags the spell from the custom spell book and drags it to the hot bar
-- @param user -
-- @param sourceId -
-- @param actionType -
-- @param actionId -
-- @param slot - 
function HandleUserActionButtonDropToHotBar(user,sourceId,actionType,actionId,slot)
	if((sourceId == "customSpellBookHandler"..this.Id) and slot ~= nil) then
		local hotbarAction = nil

		if(this:GetObjVar("BookType") == "Spellbook" ) then
			hotbarAction = GetSpellUserAction(actionId)
			hotbarAction.Slot = tonumber(slot)
			RequestAddUserActionToSlot(user,hotbarAction)
		end
	end
end

--- dispaly the custom spellbook to the user
-- @param - (game object)(user) the user game object to display the custom spellbook too
function ShowSpellBook(user)
	-- make the dynamic window transparent and dragable, and center it on the screen
	local dynamicWindow = DynamicWindow("customSpellBookHandler"..this.Id,"", 790, 486, -395, -243, "TransparentDraggable","Center")
	local school = this:GetObjVar("School") or "LightMagic"
	local spellListCount = 0
	local x = 120
	local y = 55
	dynamicWindow:AddImage(330, -68, "SkillIconsFrame", 128, 128,"Simple",UserInterfaceOptions[school].IconBorderColor)
	dynamicWindow:AddImage(358, -40, UserInterfaceOptions[school].Icon, 72, 72,"Simple",UserInterfaceOptions[school].IconColor)
	-- add the spellbook artwork
	dynamicWindow:AddImage(0, 0, "Spellbook", 789, 486,"Simple",UserInterfaceOptions[school].BookShading)
	-- loop through the spells and add their UserAction buttons
	for key,value in pairs(GetSpellListSorted()) do
		spellListCount = spellListCount + 1
		-- only 7 spells per page so switch the y value back to the top
		if(spellListCount == 8) then y = 55 end
		if(spellListCount <= 7) then
			-- LEFT PAGE
			AddUserAction(dynamicWindow,x,y,value.Key)
			dynamicWindow:AddImage(x+47,y+8,"HueSwatch",197,31,"Simple",UserInterfaceOptions[school].SpellTextBackGroundColor)
			dynamicWindow:AddImage(x+46,y,"Prestige_Divider",200,0,"Sliced","000000")
			dynamicWindow:AddImage(x+46,y+39,"Prestige_Divider",200,0,"Sliced","000000")
			dynamicWindow:AddLabel(x+60,y+12,"[" .. UserInterfaceOptions[school].SpellTextColor .. "]"..SpellData.AllSpells[value.Key].SpellDisplayName,200,30,26,"left")
			y = y + 50
		else
			-- RIGHT PAGE
			AddUserAction(dynamicWindow,x+310,y,value.Key)
			dynamicWindow:AddImage(x+357,y+8,"HueSwatch",197,31,"Simple",UserInterfaceOptions[school].SpellTextBackGroundColor)
			dynamicWindow:AddImage(x+356,y,"Prestige_Divider",200,0,"Sliced","000000")
			dynamicWindow:AddImage(x+356,y+39,"Prestige_Divider",200,0,"Sliced","000000")
			dynamicWindow:AddLabel(x+370,y+12,"[" .. UserInterfaceOptions[school].SpellTextColor .. "]" ..SpellData.AllSpells[value.Key].SpellDisplayName,200,30,26,"left")
			y = y + 50			
		end		
	end
	-- show the spellbook user interface
	-- close button
	dynamicWindow:AddButton(726, 22, "", "", 0, 0, "", "", true, "CloseSquare")
	-- show the spellbook to the player and setting this spellbook as the event handler for the dynamic window
	user:OpenDynamicWindow(dynamicWindow,this)
	-- do we play open message and sound
	if(UserInterfaceOptions[school].OpenEffectFrequency >= math.random()) then
		user:SystemMessage(UserInterfaceOptions[school].OpenMessage[math.random(#UserInterfaceOptions[school].OpenMessage)])
		user:PlayObjectSound(UserInterfaceOptions[school].OpenSound[math.random(#UserInterfaceOptions[school].OpenSound)],false)
	end
end

---------------------------------------------------------
-- EVENT HANDLERS
---------------------------------------------------------
RegisterEventHandler(EventType.ModuleAttached, GetCurrentModule(),HandleModuleAttached)
RegisterEventHandler(EventType.Message, "UseObject", HandleUseObject)
RegisterEventHandler(EventType.ClientObjectCommand, "dropAction", HandleUserActionButtonDropToHotBar)