require 'default:globals.helpers.spells'

---------------------------------------------------------
-- OVERRIDE
---------------------------------------------------------
-- Removed magic affinity
-- Made it so it knows what spell school is used
-- If you have the min skill cast success is always 100%
function CheckSpellCastSuccess(spell, mobileObj, scrollObj)
	local skillDictionary = GetSkillDictionary(mobileObj)
	local castSkillName = SpellData.AllSpells[spell].Skill
	local skillLevel = GetSkillLevel(mobileObj, castSkillName , skillDictionary) or 0
	local success = false
	-- mobs always succeed.
	if not( IsPlayerCharacter(mobileObj) ) then return true end
	-- Releasing a primed spell while stunned will result in cast failure.
	if ( HasMobileEffect(mobileObj, "Stun") ) then
		DoFizzle(mobileObj, "Cast failed.")
		return false
	end
	-- if player has the min skill to cast it, always succeed
	if(skillLevel >= SpellData.AllSpells[spell].SkillTier.Max) then success = true end
	if (success) then
		CheckSkillChance(mobileObj, castSkillName, skillLevel, GetGainChance(skillLevel))
	else
		DoFizzle(mobileObj, "Cast failed.")
	end
	return success
end

---------------------------------------------------------
-- OVERRIDE
---------------------------------------------------------
-- insert the skill check conservation on the lowering amount of reagents
function ConsumeReagents(spell, mobileObj)
    if ( SpellData.AllSpells[spell] == nil ) then return false end
    if ( SpellData.AllSpells[spell].Reagents == nil ) then
		LuaDebugCallStack("Error: Reagents is nil for spell: "..spell)
        return false
    end

	-- backpack required for reagent consume..
	local backpack = mobileObj:GetEquippedObject("Backpack")
	if not( backpack ) then return false end
	
    local minAmountOfRegs = 5
    local conservationSkillFactor = GetSkillLevel(mobileObj,"MagicFocusConservation") / 100 or 0
    minAmountOfRegs = minAmountOfRegs - math.floor(4*conservationSkillFactor)
    if(minAmountOfRegs <1) then minAmountOfRegs = 1 end

	-- consume all reagents
    for i,resourceType in pairs(SpellData.AllSpells[spell].Reagents) do
		if not( ConsumeResourceContainer(backpack, resourceType, minAmountOfRegs) ) then
			mobileObj:SendMessage("Do not have reagents.", "info")
			return false
		end
    end

	return true
end

---------------------------------------------------------
-- OVERRIDE
---------------------------------------------------------
--- Chance to gain skill
-- @param skillLevel - (number) the skill level being checked
function GetGainChance(skillLevel)
	local gainChance = 0.20
	if(skillLevel >20) then gainChance = 0.16 end
	if(skillLevel >40) then gainChance = 0.12 end
	if(skillLevel >60) then gainChance = 0.08 end
	if(skillLevel >80) then gainChance = 0.04 end
	if(skillLevel >90) then gainChance = 0.01 end
	return gainChance
end

---------------------------------------------------------
-- OVERRIDE
---------------------------------------------------------
-- insert the skill check conservation on the lowering amount of reagents
function HasReagents(spell, mobileObj, scrollObj)
    if ( SpellData.AllSpells[spell] == nil ) then return false end
    -- mobs don't use reagents
	if not( mobileObj:IsPlayer() ) then return true end
    if(scrollObj) then return false end

    if ( SpellData.AllSpells[spell].Reagents == nil ) then
		LuaDebugCallStack("Error: Reagents is nil for spell: "..spell)
        return false
    end
    local backpack = mobileObj:GetEquippedObject("Backpack")
    if ( backpack ~= nil ) then
        local have = 0
        local minAmountOfRegs = 5
        local conservationSkillFactor = GetSkillLevel(mobileObj,"MagicFocusConservation") / 100 or 0
        minAmountOfRegs = minAmountOfRegs - math.floor(4*conservationSkillFactor)
        if(minAmountOfRegs <1) then minAmountOfRegs = 1 end
        for i,resourceType in ipairs(SpellData.AllSpells[spell].Reagents) do
            if ( resourceType ~= nil and CountResourcesInContainer(backpack,resourceType) >= minAmountOfRegs) then
                have = have + 1
            end
        end
        if ( have == #SpellData.AllSpells[spell].Reagents ) then
            return true
        end
    end
	return false
end

---------------------------------------------------------
-- OVERRIDE
---------------------------------------------------------
-- Always allow players to cast spells, the icon wouldnt appear if they couldnt cast it.
function HasSpell(spell, mobileObj, scrollObj)
	return true
end

---------------------------------------------------------
-- CUSTOM PROJECTILE EFFECT
---------------------------------------------------------
-- Allows me to bypass projectile effects and use this in
-- my mobile effect spells instead.
function DoProjectileSpellEffects(spellCaster,spellTarget,spellName,rootMobileEffect)
	local spellFXName = SpellData.AllSpells[spellName].SpellTravelEffect
	local spellFXArgs = SpellData.AllSpells[spellName].SpellTravelEffectArgs
	local spellHitFX = SpellData.AllSpells[spellName].SpellHitEffect
	local spellHitFXDuration = SpellData.AllSpells[spellName].SpellHitEffectDuration or 2
	local distance = spellTarget:DistanceFrom(spellCaster)
	local overrideRate = 10
	local timer = (distance - .5) / overrideRate
	local spellTime = timer * 1000
	spellCaster:PlayProjectileEffectTo(spellFXName,spellTarget,overrideRate,timer,spellFXArgs)

	CallFunctionDelayed(TimeSpan.FromMilliseconds(spellTime), function()
		-- check for root mobile function calls for projectile spells 
		if(rootMobileEffect.DoDelayedDamage) then
			rootMobileEffect.DoDelayedDamage(rootMobileEffect)
			if(spellHitFX) then spellTarget:PlayEffect(spellHitFX,spellHitFXDuration) end
			return
		end 
	end)	
end