-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.AirElemental = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	CategoryKey = "AirMagic",
	MobTableKey = "AirElemental",
	SummonedDuration = SpellData.SpellEffectDuration.Summoning,
	MaxBonusSummonedDurationInMinutes = 2,
	SpawnRadius = 5,
	BaseAttack = 10,
	MaxAttackBonus = 20,
	BaseArmor = 30,
	MaxArmorBonus = 30,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
        RegisterSingleEventHandler(EventType.CreatedObject, "AirElementalCreated", function(success, airElemental,castingArgs)
            self.HandleCreatedAirElementalEvent(success,airElemental,castingArgs)
        end)
		self.SummonAirElemental(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	SummonAirElemental = function(self,root,target,args)
		local castingArgs =
		{
			Caster = self.ParentObj,
			CategoryKey = self.CategoryKey,
			MobTableKey = self.MobTableKey,
			SummonedDuration = self.SummonedDuration,
			MaxBonusSummonedDurationInMinutes = self.MaxBonusSummonedDurationInMinutes,
			SpawnRadius = self.SpawnRadius,
			BaseAttack = self.BaseAttack,
			MaxAttackBonus = self.MaxAttackBonus,
			BaseArmor = self.BaseArmor,
			MaxArmorBonus = self.MaxArmorBonus,
		}
		if(CanAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey)) then
			CreateObj("summoned_air_elemental_1", GetRandomPassableLocationInRadius(self.ParentObj:GetLoc(),self.SpawnRadius,true),"AirElementalCreated",castingArgs)	
		end
	end,
	HandleCreatedAirElementalEvent = function(success,airElemental,castingArgs)
		local summonFactor = (GetSkillLevel(castingArgs.Caster,"MagicFocusSummoning")/100)
		local durationBonus = summonFactor * castingArgs.MaxBonusSummonedDurationInMinutes
		local duration = castingArgs.SummonedDuration + TimeSpan.FromMinutes(durationBonus)
		local attackBonus = summonFactor * castingArgs.MaxAttackBonus
		local attack = castingArgs.BaseAttack + attackBonus
		local armorBonus = summonFactor * castingArgs.MaxArmorBonus
		local armor = castingArgs.BaseArmor + attackBonus
		-- add summon information to object
		airElemental:SetObjVar("Summoned",true)
		airElemental:SetObjVar("SummonMaster",castingArgs.Caster)
		-- attach the delete timer for summoned objects
		airElemental:AddModule("summoned_delete_timer",{Duration=duration})
		-- key information for the summoning system
		airElemental:SetObjVar("CategoryKey",castingArgs.CategoryKey)
		airElemental:SetObjVar("MobileTableKey",castingArgs.MobTableKey)
		airElemental:SetSharedObjectProperty("Title", "-| " .. castingArgs.Caster:GetName() .. "'s Minion |-")
		-- scale air elementals power
		airElemental:SetObjVar("Attack",attack)
		airElemental:SetObjVar("Attack",armor)
		-- play effects
		PlayEffectAtLoc("CloakEffect",airElemental:GetLoc())
		PlayEffectAtLoc("PrimedAir",airElemental:GetLoc(),2.5)
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey,airElemental)
	end,
}