-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.AirMephit = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	CategoryKey = "AirMagic",
	MobTableKey = "AirMephit",
	SummonedDuration = SpellData.SpellEffectDuration.Summoning,
	MaxBonusSummonedDurationInMinutes = 3,
	SpawnRadius = 2,
	BaseAttack = 3,
	MaxAttackBonus = 12,
	BaseArmor = 10,
	MaxArmorBonus = 30,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
        RegisterSingleEventHandler(EventType.CreatedObject, "AirMephitCreated", function(success, airMephit,castingArgs)
            self.HandleCreateAirMephitEvent(success,airMephit,castingArgs)
        end)
		self.SummonAirMephit(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	SummonAirMephit = function(self,root,target,args)
		local castingArgs =
		{
			Caster = self.ParentObj,
			CategoryKey = self.CategoryKey,
			MobTableKey = self.MobTableKey,
			SummonedDuration = self.SummonedDuration,
			MaxBonusSummonedDurationInMinutes = self.MaxBonusSummonedDurationInMinutes,
			SpawnRadius = self.SpawnRadius,
			BaseAttack = self.BaseAttack,
			MaxAttackBonus = self.MaxAttackBonus,
			BaseArmor = self.BaseArmor,
			MaxArmorBonus = self.MaxArmorBonus,
		}
		if(CanAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey)) then
			CreateObj("summoned_air_mephit_1", GetRandomPassableLocationInRadius(self.ParentObj:GetLoc(),self.SpawnRadius,true),"AirMephitCreated",castingArgs)	
		end
	end,
	HandleCreateAirMephitEvent = function(success,airMephit,castingArgs)
		local summonFactor = (GetSkillLevel(castingArgs.Caster,"MagicFocusSummoning")/100)
		local durationBonus = summonFactor * castingArgs.MaxBonusSummonedDurationInMinutes
		local duration = castingArgs.SummonedDuration + TimeSpan.FromMinutes(durationBonus)
		local attackBonus = summonFactor * castingArgs.MaxAttackBonus
		local attack = castingArgs.BaseAttack + attackBonus
		local armorBonus = summonFactor * castingArgs.MaxArmorBonus
		local armor = castingArgs.BaseArmor + attackBonus
		-- add summon information to object
		airMephit:SetObjVar("Summoned",true)
		airMephit:SetObjVar("SummonMaster",castingArgs.Caster)
		-- attach the delete timer for summoned objects
		airMephit:AddModule("summoned_delete_timer",{Duration=duration})
		-- key information for the summoning system
		airMephit:SetObjVar("CategoryKey",castingArgs.CategoryKey)
		airMephit:SetObjVar("MobileTableKey",castingArgs.MobTableKey)
		airMephit:SetSharedObjectProperty("Title", "-| " .. castingArgs.Caster:GetName() .. "'s Minion |-")
		-- scale air mephits power
		airMephit:SetObjVar("Attack",attack)
		airMephit:SetObjVar("Attack",armor)
		-- play effects
		PlayEffectAtLoc("CloakEffect",airMephit:GetLoc())
		PlayEffectAtLoc("PrimedAir",airMephit:GetLoc(),2.5)
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey,airMephit)
	end,
}