-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- TODO: 
-- Duration Scaling
-- * base duration is 5 minutes
-- * bonuse duration 1 - 15 minutes based on magic focus enhancment
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.ForceShield = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Caster = nil,
	Duration = SpellData.SpellEffectDuration.BuffShort,
	BashingDamageReductionTimes = 0.25,
	BowDamageReductionTimes = 0.25,
	MaxDurationBuffInSeconds = 15,
	PiercingDamageReductionTimes = 0.25,
	SlashingDamageReductionTimes = 0.25,
	Target = nil,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		self.Caster = target
		self.Target = self.ParentObj
		local enhancementSkillFactor = GetSkillLevel(self.Caster,"MagicFocusEnhancement") / 100 
		local bonusEnhancementDuration = enhancementSkillFactor * self.MaxDurationBuffInSeconds
		local totalPiercingDamageReductionTimes = enhancementSkillFactor * self.PiercingDamageReductionTimes
		local totalSlashingDamageReductionTimes = enhancementSkillFactor * self.SlashingDamageReductionTimes
		local totalBashingDamageReductionTimes = enhancementSkillFactor * self.BashingDamageReductionTimes
		local totalBowDamageReductionTimes = enhancementSkillFactor * self.BowDamageReductionTimes
		-- set minimums
		if(totalPiercingDamageReductionTimes < 0.05) then totalPiercingDamageReductionTimes = 0.05 end
		if(totalSlashingDamageReductionTimes < 0.05) then totalSlashingDamageReductionTimes = 0.05 end
		if(totalBashingDamageReductionTimes < 0.05) then totalBashingDamageReductionTimes = 0.05 end
		if(totalBowDamageReductionTimes < 0.05) then totalBowDamageReductionTimes = 0.05 end
		self.Duration = self.Duration + TimeSpan.FromSeconds(bonusEnhancementDuration)
		-- apply buff icons
		if ( self.Target:IsPlayer() ) then
			AddBuffIcon(self.Target, "ForceShield", "Force Shield", "Shock Wave", "a translucent shield appears around you.")
			SetMobileMod(self.Target, "BashingDamageReductionTimes", "ForceShield", totalPiercingDamageReductionTimes)
			SetMobileMod(self.Target, "SlashingDamageReductionTimes", "ForceShield", totalSlashingDamageReductionTimes)
			SetMobileMod(self.Target, "PiercingDamageReductionTimes", "ForceShield", totalBashingDamageReductionTimes)
			SetMobileMod(self.Target, "BowDamageReductionTimes", "ForceShield", totalBowDamageReductionTimes)
		end
		self.PlayEffects(self,root,target,args)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.Target:IsPlayer() ) then
			RemoveBuffIcon(self.Target, "ForceShield")
			SetMobileMod(self.Target, "BashingDamageReductionTimes", "ForceShield", nil)
			SetMobileMod(self.Target, "SlashingDamageReductionTimes", "ForceShield", nil)
			SetMobileMod(self.Target, "PiercingDamageReductionTimes", "ForceShield", nil)
			SetMobileMod(self.Target, "BowDamageReductionTimes", "ForceShield", nil)
			self.Target:SystemMessage("[FFFF77]force shield dissapates...")
			self.Caster:SystemMessage("[FFFF77]your force shield on " .. self.Target:GetName() .. " has vanished...")
		end
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.Target:PlayObjectSound("event:/magic/misc/magic_water_restoration",false)
		self.Target:PlayEffectWithArgs("BubbleEnergyEffect",3.25,"Bone=Head")
		self.Target:PlayEffectWithArgs("YellowPortalEffect",3,"Bone=Ground")
	end,
}