-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Damage Scaling
-- * base damage is 15 - 25
-- * bonus damage is 1 - 50 based on magic focus enhancement
-- Range Scaling
-- * base range diameter is 4 units
-- * bonus range is 1 - 5 based on magic focus enhancement
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.StormCall = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	Range = 3,
	MinDamage = 15,
	MaxDamage = 25,
	Caster = nil,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		self.Caster = self.ParentObj
		self.PlayEffects(self,root,target,args)
		CallFunctionDelayed(TimeSpan.FromSeconds(0.25),function() self.DoAoEDamage(self,root,target,args) end)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		PlayEffectAtLoc("LightningEffect",self.Caster:GetLoc(),0)
		PlayEffectAtLoc("CastAir2",self.Caster:GetLoc(),2.5)
		local angle = 0
		local x, y, r = self.Caster:GetLoc().X, self.Caster:GetLoc().Z, self.Range
		for i = 45, 360, 45 do
		  local angle = i * math.pi / 180
		  local ptx, pty = x + r * math.cos( angle ), y + r * math.sin( angle )
		  PlayEffectAtLoc("CastAir2",Loc(ptx,0,pty),2.5)
		  PlayEffectAtLoc("LightningExplosionEffect",Loc(ptx,1.5,pty),3.25)
		  PlayEffectAtLoc("LightningCloudEffect",Loc(ptx,0,pty),0)
		end
	end,
	DoAoEDamage = function(self,root,target,args)
		local spellCaster = target
		local enhancementSkill = GetSkillLevel(spellCaster,"MagicFocusEnhancement")
		local bonusEnhancementDamage = enhancementSkill / 2.25
		local bonusEnhancementRange = enhancementSkill / 25
    	-- no demons or undead
    	local mobiles = FindObjects(SearchMobileInRange(self.Range + bonusEnhancementRange))
		for i=1,#mobiles do
			local spellDamage = math.random(self.MinDamage,self.MaxDamage) + bonusEnhancementDamage
			if (mobiles[i]:HasLineOfSightToLoc(mobiles[i]:GetLoc(),ServerSettings.Combat.LOSEyeLevel) and ValidCombatTarget(spellCaster, mobiles[i], true)) then
				mobiles[i]:PlayEffect("VoidExplosionEffect")
				mobiles[i]:PlayEffect("BlacksmithSpark")
				mobiles[i]:PlayEffect("BlacksmithSpark2")
				mobiles[i]:SendMessage("DamageInflicted",spellCaster,spellDamage,"MAGIC",false,false,false,spellCaster)
			end
		end				
	end,
}