-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- TODO: 
-- Duration Scaling
-- * base duration is 5 minutes
-- * bonuse duration 1 - 15 minutes based on magic focus enhancment
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.Velocity = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Caster = nil,
	Duration = SpellData.SpellEffectDuration.BuffNormal,
	MaxAttackSpeedTimes = 0.2,
	MaxDurationBuffInMinutes = 15,
	MaxMovementSpeedPlus = 0.2,
	MaxStaminaRegenPlus = 0.5,
	Target = nil,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		self.Caster = target
		self.Target = self.ParentObj
		local enhancementSkillFactor = GetSkillLevel(self.Caster,"MagicFocusEnhancement") / 100 
		local bonusEnhancementDuration = enhancementSkillFactor * self.MaxDurationBuffInMinutes
		local totalMoveSpeedPlus = enhancementSkillFactor * self.MaxMovementSpeedPlus
		local totalStaminaRegenPlus = enhancementSkillFactor * self.MaxStaminaRegenPlus
		local totalAttackSpeedTimes = enhancementSkillFactor * self.MaxAttackSpeedTimes
		-- set minimums
		if(totalMoveSpeedPlus < 0.025) then totalMoveSpeedPlus = 0.025 end
		if(totalStaminaRegenPlus < 0.1) then totalStaminaRegenPlus = 0.1 end
		if(totalAttackSpeedTimes < 0.025) then totalAttackSpeedTimes = 0.025 end
		self.Duration = self.Duration + TimeSpan.FromMinutes(bonusEnhancementDuration)
		-- apply buff icons
		if ( self.Target:IsPlayer() ) then
			AddBuffIcon(self.Target, "Velocity", "Velocity", "stanceability", "you feel a raging storm of power inside you.")
			SetMobileMod(self.Target, "MoveSpeedPlus", "Velocity", totalMoveSpeedPlus)
			SetMobileMod(self.Target, "StaminaRegenPlus", "Velocity", totalStaminaRegenPlus)
			SetMobileMod(self.Target, "AttackSpeedTimes", "Velocity", totalAttackSpeedTimes)
		end
		self.PlayEffects(self,root,target,args)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.Target:IsPlayer() ) then
			RemoveBuffIcon(self.Target, "Velocity")
			SetMobileMod(self.Target, "MoveSpeedPlus", "Velocity", nil)
			SetMobileMod(self.Target, "StaminaRegenPlus", "Velocity", nil)
			SetMobileMod(self.Target, "AttackSpeedTimes", "Velocity", nil)
			self.Target:SystemMessage("[FFFF77]volicty leaves you... you begin to slow down...")
			self.Caster:SystemMessage("[FFFF77]your velocity on " .. self.Target:GetName() .. " has dwindled...")
		end
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.Target:PlayObjectSound("event:/magic/misc/magic_water_restoration",false)
		self.Target:PlayEffectWithArgs("CastAir",2.75,"Bone=Head")
		self.Target:PlayEffectWithArgs("YellowPortalEffect",3.5,"Bone=Ground")
	end,
}