-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.BoneWarrior = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	CategoryKey = "DarkMagic",
	MobTableKey = "BoneWarriors",
	SummonedDuration = SpellData.SpellEffectDuration.Summoning,
	MaxBonusSummonedDurationInMinutes = 5,
	SpawnRadius = 5,
	BaseAttack = 5,
	MaxAttackBonus = 15,
	BaseArmor = 20,
	MaxArmorBonus = 40,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
        RegisterSingleEventHandler(EventType.CreatedObject, "BoneWarriorCreated", function(success, boneWarrior,castingArgs)
            self.HandleCreateBoneWarriorEvent(success,boneWarrior,castingArgs)
        end)
		self.SummonBoneWarrior(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	SummonBoneWarrior = function(self,root,target,args)
		local castingArgs =
		{
			Caster = self.ParentObj,
			CategoryKey = self.CategoryKey,
			MobTableKey = self.MobTableKey,
			SummonedDuration = self.SummonedDuration,
			MaxBonusSummonedDurationInMinutes = self.MaxBonusSummonedDurationInMinutes,
			SpawnRadius = self.SpawnRadius,
			BaseAttack = self.BaseAttack,
			MaxAttackBonus = self.MaxAttackBonus,
			BaseArmor = self.BaseArmor,
			MaxArmorBonus = self.MaxArmorBonus,
		}
		if(CanAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey)) then
			CreateObj("summoned_bone_warrior_1", GetRandomPassableLocationInRadius(self.ParentObj:GetLoc(),self.SpawnRadius,true),"BoneWarriorCreated",castingArgs)	
		end
	end,
	HandleCreateBoneWarriorEvent = function(success,boneWarrior,castingArgs)
		local summonFactor = (GetSkillLevel(castingArgs.Caster,"MagicFocusSummoning")/100)
		local durationBonus = summonFactor * castingArgs.MaxBonusSummonedDurationInMinutes
		local duration = castingArgs.SummonedDuration + TimeSpan.FromMinutes(durationBonus)
		local attackBonus = summonFactor * castingArgs.MaxAttackBonus
		local attack = castingArgs.BaseAttack + attackBonus
		local armorBonus = summonFactor * castingArgs.MaxArmorBonus
		local armor = castingArgs.BaseArmor + attackBonus
		-- add summon information to object
		boneWarrior:SetObjVar("Summoned",true)
		boneWarrior:SetObjVar("SummonMaster",castingArgs.Caster)
		-- attach the delete timer for summoned objects
		boneWarrior:AddModule("summoned_delete_timer",{Duration=duration})
		-- key information for the summoning system
		boneWarrior:SetObjVar("CategoryKey",castingArgs.CategoryKey)
		boneWarrior:SetObjVar("MobileTableKey",castingArgs.MobTableKey)
		boneWarrior:SetSharedObjectProperty("Title", "-| " .. castingArgs.Caster:GetName() .. "'s Minion |-")
		-- scale skeletons power
		boneWarrior:SetObjVar("Attack",attack)
		boneWarrior:SetObjVar("Attack",armor)
		PlayEffectAtLoc("CloakEffect",boneWarrior:GetLoc())
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey,boneWarrior)
	end,
}