-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.CloakOfShadows = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	MaxLifeLoss = 100,
	MaxMagicDamageReduction = 10,
	MaxMagicDamageReductionBonus = 30,
	MaxBonusDurationBonusInMinutes = 15,
	MaxBonusMana = 30,
	MaxBonusStamina = 30,
	Duration = SpellData.SpellEffectDuration.BuffLong,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args) 
		local enhancementFactor = (GetSkillLevel(target,"MagicFocusEnhancement")/100)
		local bonusEnhancementDuration = enhancementFactor * self.MaxBonusDurationBonusInMinutes
		local magicDamageReduction = enhancementFactor * self.MaxMagicDamageReduction
		local bonusMagicDamageReduction = enhancementFactor * self.MaxMagicDamageReductionBonus
		local bonusMana = enhancementFactor * self.MaxBonusMana
		local bonusStamina = enhancementFactor * self.MaxBonusStamina
		local totalMagicDamageReduction = magicDamageReduction + bonusMagicDamageReduction
		local totalLifeLoss = -(enhancementFactor * self.MaxLifeLoss)

		-- set minimums
		if(bonusMana < 5) then bonusMana = 5 end
		if(bonusStamina < 5) then bonusStamina = 5 end
		if(totalMagicDamageReduction < 5) then totalMagicDamageReduction = 5 end
		if(totalLifeLoss > -20) then totalLifeLoss = -20 end

		self.Duration = self.Duration + TimeSpan.FromMinutes(bonusEnhancementDuration)		
		-- apply buff icons
		if ( self.ParentObj:IsPlayer() ) then
			AddBuffIcon(self.ParentObj, "CloakOfShadows", "Cloak of Shadows", "Phantom", "your sacrifice gives you strength.")
		end
		-- set mobile mods
		SetMobileMod(self.ParentObj, "MagicDamageReductionTimes", "CloakOfShadows", bonusMagicDamageReduction + magicDamageReduction)
		SetMobileMod(self.ParentObj, "MaxHealthPlus", "CloakOfShadows", totalLifeLoss)
		SetMobileMod(self.ParentObj, "MaxManaPlus", "CloakOfShadows", bonusMana)
		SetMobileMod(self.ParentObj, "MaxStaminaPlus", "CloakOfShadows", bonusStamina)
		-- play spell effects
		self.PlayEffects(self)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.ParentObj:IsPlayer() ) then
			RemoveBuffIcon(self.ParentObj, "CloakOfShadows")
			self.ParentObj:SystemMessage("[AA5555]The power of your cloak fades...")
		end
		SetMobileMod(self.ParentObj, "MagicDamageReductionTimes", "CloakOfShadows", nil)
		SetMobileMod(self.ParentObj, "MaxHealthPlus", "CloakOfShadows", nil)
		SetMobileMod(self.ParentObj, "MaxManaPlus", "CloakOfShadows", nil)
		SetMobileMod(self.ParentObj, "MaxStaminaPlus", "CloakOfShadows", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.ParentObj:PlayObjectSound("event:/monsters/phantom/phantom_death",false)
		self.ParentObj:PlayEffectWithArgs("VoidAuraEffect",6,"Bone=Ground")
		self.ParentObj:PlayEffectWithArgs("PurplePortalEffect",3,"Bone=Ground")
		self.ParentObj:SystemMessage("[AA5555]As your flesh shrivels tight ... you feel your power grow.")	
	end,
}