-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.CursedSoul = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	MaxHealthRegenLoss = 0.8,
	MaxManaRegenLoss = 0.6,
	MaxStaminaRegenLoss = 1,
	MaxMovementSpeedLoss = 0.25,
	MinMovementSpeedLoss = 0.05,
	MinRegenLoss = 0.1,

	Duration = SpellData.SpellEffectDuration.DebuffNormal,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		local enhancementSkillFactor = GetSkillLevel(target,"MagicFocusEnhancement")/100 
		local healthRegenLoss = enhancementSkillFactor * self.MaxHealthRegenLoss
		local manaRegenLoss = enhancementSkillFactor * self.MaxManaRegenLoss
		local staminaRegenLoss = enhancementSkillFactor * self.MaxStaminaRegenLoss
		local movementSpeedLoss = enhancementSkillFactor * self.MaxMovementSpeedLoss

		-- scale to minimums
		if(healthRegenLoss < self.MinRegenLoss) then healthRegenLoss = self.MinRegenLoss end
		if(manaRegenLoss < self.MinRegenLoss) then manaRegenLoss = self.MinRegenLoss end
		if(staminaRegenLoss < self.MinRegenLoss) then staminaRegenLoss = self.MinRegenLoss end
		if(movementSpeedLoss < self.MinMovementSpeedLoss) then movementSpeedLoss = self.MinMovementSpeedLoss end

		self.Duration = self.Duration + TimeSpan.FromSeconds(bonusEnhancementDuration)		
		-- apply buff icons
		if ( self.ParentObj:IsPlayer() ) then
			AddBuffIcon(self.ParentObj, "CursedSoul", "[AA5555]Cursed Soul[-]", "voidblast", "You feel weak and unsure with your connection to this world.")
		end
		-- set mobile mods
		SetMobileMod(self.ParentObj, "HealthRegenPlus", "CursedSoul", -healthRegenLoss)
		SetMobileMod(self.ParentObj, "ManaRegenPlus", "CursedSoul", -manaRegenLoss)
		SetMobileMod(self.ParentObj, "StaminaRegenPlus", "CursedSoul", -staminaRegenLoss)
		SetMobileMod(self.ParentObj, "MoveSpeedPlus", "CursedSoul", -movementSpeedLoss)
		-- play spell effects
		self.PlayEffects(self,root,target,args)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.ParentObj:IsPlayer() ) then
			RemoveBuffIcon(self.ParentObj, "CursedSoul")
		end
		SetMobileMod(self.ParentObj, "HealthRegenPlus", "CursedSoul", nil)
		SetMobileMod(self.ParentObj, "ManaRegenPlus", "CursedSoul", nil)
		SetMobileMod(self.ParentObj, "StaminaRegenPlus", "CursedSoul", nil)
		SetMobileMod(self.ParentObj, "MoveSpeedPlus", "CursedSoul", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.ParentObj:PlayObjectSound("event:/monsters/phantom/phantom_death",false)
		self.ParentObj:PlayEffectWithArgs("VoidExplosionEffect",2.5,"Bone=Ground")
		self.ParentObj:PlayEffectWithArgs("PurplePortalEffect",3,"Bone=Ground")
		if(target:IsPlayer()) then
			target:SystemMessage("[AA5555]you curse " .. target:GetName())	
		end
	end,
}