-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Duration Scaling
-- * base duration is 2 minutes
-- * bonuse duration 1 - 8 minutes based on magic focus enhancment
-- Bonus Health Scaling 
-- * base health bonus is 50
-- * bonus health is 1 - 150 based on magic focus enhancment
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.DemonicWard = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.BuffLong,
	MaxDemonDamageReduction = 20,
	MaxBonusDemonDamageReduction = 30,
	MaxBonusDuration = 15,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		local darkMagicSkillFactor = GetSkillLevel(target,"DarkMagic")/100
		local enhancementSkillFactor = GetSkillLevel(target,"MagicFocusEnhancement")/100
		local demonDamageReduction = darkMagicSkillFactor * self.MaxDemonDamageReduction
		local bonusDemonDamageRecution = enhancementSkillFactor * self.MaxBonusDemonDamageReduction
		local totalReduction = demonDamageReduction + bonusDemonDamageRecution
		local bonusDuration = enhancementSkillFactor * self.MaxBonusDuration 
		self.Duration = self.Duration + TimeSpan.FromMinutes(bonusDuration)	
		-- apply buff icons
		if ( self.ParentObj:IsPlayer() ) then
			AddBuffIcon(self.ParentObj, "DemonicWard", "Demonic Ward", "Arcane Shield2", "darkness surrounds your soul.")
		end
		SetMobileMod(self.ParentObj, "DemonDamageReductionTimes", "DemonicWard", totalReduction)
		self.PlayEffects(self)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.ParentObj:IsPlayer() ) then
			RemoveBuffIcon(self.ParentObj, "DemonicWard")
			self.ParentObj:SystemMessage("[FFFF77]Your demonic shroud has left you...")
		end
		SetMobileMod(self.ParentObj, "DemonDamageReductionTimes", "DemonicWard", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.ParentObj:PlayObjectSound("event:/monsters/phantom/phantom_death",false)
		self.ParentObj:PlayEffectWithArgs("VoidAuraEffect",4,"Bone=Ground")
		self.ParentObj:PlayEffectWithArgs("CastVoid",3,"Bone=Head")
	end,
}