-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.FireElemental = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	CategoryKey = "FireMagic",
	MobTableKey = "FireElemental",
	SummonedDuration = SpellData.SpellEffectDuration.Summoning,
	MaxBonusSummonedDurationInMinutes = 2,
	SpawnRadius = 5,
	BaseAttack = 15,
	MaxAttackBonus = 22,
	BaseArmor = 20,
	MaxArmorBonus = 25,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
        RegisterSingleEventHandler(EventType.CreatedObject, "FireElementalCreated", function(success, fireElemental,castingArgs)
            self.HandleCreatedFireElementalEvent(success,fireElemental,castingArgs)
        end)
		self.SummonFireElemental(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	SummonFireElemental = function(self,root,target,args)
		local castingArgs =
		{
			Caster = self.ParentObj,
			CategoryKey = self.CategoryKey,
			MobTableKey = self.MobTableKey,
			SummonedDuration = self.SummonedDuration,
			MaxBonusSummonedDurationInMinutes = self.MaxBonusSummonedDurationInMinutes,
			SpawnRadius = self.SpawnRadius,
			BaseAttack = self.BaseAttack,
			MaxAttackBonus = self.MaxAttackBonus,
			BaseArmor = self.BaseArmor,
			MaxArmorBonus = self.MaxArmorBonus,
		}
		if(CanAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey)) then
			CreateObj("summoned_fire_elemental_1", GetRandomPassableLocationInRadius(self.ParentObj:GetLoc(),self.SpawnRadius,true),"FireElementalCreated",castingArgs)	
		end
	end,
	HandleCreatedFireElementalEvent = function(success,fireElemental,castingArgs)
		local summonFactor = (GetSkillLevel(castingArgs.Caster,"MagicFocusSummoning")/100)
		local durationBonus = summonFactor * castingArgs.MaxBonusSummonedDurationInMinutes
		local duration = castingArgs.SummonedDuration + TimeSpan.FromMinutes(durationBonus)
		local attackBonus = summonFactor * castingArgs.MaxAttackBonus
		local attack = castingArgs.BaseAttack + attackBonus
		local armorBonus = summonFactor * castingArgs.MaxArmorBonus
		local armor = castingArgs.BaseArmor + attackBonus
		-- add summon information to object
		fireElemental:SetObjVar("Summoned",true)
		fireElemental:SetObjVar("SummonMaster",castingArgs.Caster)
		-- attach the delete timer for summoned objects
		fireElemental:AddModule("summoned_delete_timer",{Duration=duration})
		-- key information for the summoning system
		fireElemental:SetObjVar("CategoryKey",castingArgs.CategoryKey)
		fireElemental:SetObjVar("MobileTableKey",castingArgs.MobTableKey)
		fireElemental:SetSharedObjectProperty("Title", "-| " .. castingArgs.Caster:GetName() .. "'s Minion |-")
		-- scale fire elementals power
		fireElemental:SetObjVar("Attack",attack)
		fireElemental:SetObjVar("Attack",armor)
		-- play effects
		PlayEffectAtLoc("CloakEffect",fireElemental:GetLoc())
		PlayEffectAtLoc("PrimedAir",fireElemental:GetLoc(),2.5)
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey,fireElemental)
	end,
}