-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
--TODO: DOCUMENT SCALING
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.FireMephit = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.Instant,
	CategoryKey = "FireMagic",
	MobTableKey = "FireMephit",
	SummonedDuration = SpellData.SpellEffectDuration.Summoning,
	MaxBonusSummonedDurationInMinutes = 3,
	SpawnRadius = 3,
	BaseAttack = 4,
	MaxAttackBonus = 10,
	BaseArmor = 12,
	MaxArmorBonus = 22,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
        RegisterSingleEventHandler(EventType.CreatedObject, "FireMephitCreated", function(success, fireMephit,castingArgs)
            self.HandleCreateFireMephitEvent(success,fireMephit,castingArgs)
        end)
		self.SummonFireMephit(self,root,target,args)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	SummonFireMephit = function(self,root,target,args)
		local castingArgs =
		{
			Caster = self.ParentObj,
			CategoryKey = self.CategoryKey,
			MobTableKey = self.MobTableKey,
			SummonedDuration = self.SummonedDuration,
			MaxBonusSummonedDurationInMinutes = self.MaxBonusSummonedDurationInMinutes,
			SpawnRadius = self.SpawnRadius,
			BaseAttack = self.BaseAttack,
			MaxAttackBonus = self.MaxAttackBonus,
			BaseArmor = self.BaseArmor,
			MaxArmorBonus = self.MaxArmorBonus,
		}
		if(CanAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey)) then
			CreateObj("summoned_fire_mephit_1", GetRandomPassableLocationInRadius(self.ParentObj:GetLoc(),self.SpawnRadius,true),"FireMephitCreated",castingArgs)	
		end
	end,
	HandleCreateFireMephitEvent = function(success,fireMephit,castingArgs)
		local summonFactor = (GetSkillLevel(castingArgs.Caster,"MagicFocusSummoning")/100)
		local durationBonus = summonFactor * castingArgs.MaxBonusSummonedDurationInMinutes
		local duration = castingArgs.SummonedDuration + TimeSpan.FromMinutes(durationBonus)
		local attackBonus = summonFactor * castingArgs.MaxAttackBonus
		local attack = castingArgs.BaseAttack + attackBonus
		local armorBonus = summonFactor * castingArgs.MaxArmorBonus
		local armor = castingArgs.BaseArmor + attackBonus
		-- add summon information to object
		fireMephit:SetObjVar("Summoned",true)
		fireMephit:SetObjVar("SummonMaster",castingArgs.Caster)
		-- attach the delete timer for summoned objects
		fireMephit:AddModule("summoned_delete_timer",{Duration=duration})
		-- key information for the summoning system
		fireMephit:SetObjVar("CategoryKey",castingArgs.CategoryKey)
		fireMephit:SetObjVar("MobileTableKey",castingArgs.MobTableKey)
		fireMephit:SetSharedObjectProperty("Title", "-| " .. castingArgs.Caster:GetName() .. "'s Minion |-")
		-- scale fire mephits power
		fireMephit:SetObjVar("Attack",attack)
		fireMephit:SetObjVar("Attack",armor)
		-- play effects
		PlayEffectAtLoc("CloakEffect",fireMephit:GetLoc())
		PlayEffectAtLoc("PrimedAir",fireMephit:GetLoc(),2.5)
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(castingArgs.Caster,castingArgs.CategoryKey,castingArgs.MobTableKey,fireMephit)
	end,
}