-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Duration Scaling
-- * base duration is 10 seconds
-- * bonuse duration 1 - 20 seconds based on magic focus enhancment
-- base attack bonus
-- * max attack bonus is 10%
-- * bonus base attack is 1%-20% based on magic focus enhancment
-- attack speed bonus
-- * max attack speed is 30%
-- * bonus is 1%-30% based on magic focus enhancement
MobileEffectLibrary.DivineFury = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	BaseAttackTimes = 0.1,
	Duration = SpellData.SpellEffectDuration.BuffVeryShort,
	MaxAttackTimes = 0.2,
	MaxAttackSpeedTimes = 30,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		local enhancementSkill = GetSkillLevel(target,"MagicFocusEnhancement") 
		local bonusEnhancementDuration = (enhancementSkill / 100) * 20		
		local bonusAttackTimes = enhancementSkill/100 * self.MaxAttackTimes
		local bonusAttackSpeedTimes = 1 + ((enhancementSkill/100 * self.MaxAttackSpeedTimes) / 100)
		self.Duration = self.Duration + TimeSpan.FromSeconds(bonusEnhancementDuration)		
		if ( self.ParentObj:IsPlayer() ) then
			AddBuffIcon(self.ParentObj, "DivineFury", "Divine Fury", "Ignite", "The power of the Gods flow through you.")
		end
		SetMobileMod(self.ParentObj, "AttackTimes", "DivineFuryAttackTimes",self.BaseAttackTimes + bonusAttackTimes)
		SetMobileMod(self.ParentObj, "AttackSpeedTimes", "DivineFuryAttackSpeedTimes", bonusAttackSpeedTimes)
		self.PlayEffects(self)
	end,
	OnExitState = function(self,root)
		if ( self.ParentObj:IsPlayer() ) then
			RemoveBuffIcon(self.ParentObj, "DivineFury")
			self.ParentObj:SystemMessage("[FFFF77]The light of divine fury leaves you...")
		end
		SetMobileMod(self.ParentObj, "AttackTimes", "DivineFuryAttackTimes", nil)
		SetMobileMod(self.ParentObj, "AttackSpeedTimes", "DivineFuryAttackSpeedTimes", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.ParentObj:PlayObjectSound("event:/magic/misc/magic_water_restoration",false)
		self.ParentObj:PlayEffectWithArgs("HolyEffect",0,"Bone=Ground")
		self.ParentObj:PlayEffectWithArgs("YellowPortalEffect",2.5,"Bone=Ground")
		self.ParentObj:SystemMessage("[FFFF77]Embracing your teachings in your faith your ready to destroy darkness...")	
	end,
}