-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Duration Scaling
-- * base duration is 2 minutes
-- * bonuse duration 1 - 8 minutes based on magic focus enhancment
-- Bonus Health Scaling 
-- * base health bonus is 50
-- * bonus health is 1 - 150 based on magic focus enhancment
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.Heroism = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	Duration = SpellData.SpellEffectDuration.BuffNormal,
	BaseBonusHealth = 50,
	MaxBonusHealth = 150,
	Caster = nil,
	Target = nil,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		self.Caster = target
		self.Target = self.ParentObj		
		local enhancementSkill = GetSkillLevel(self.Caster,"MagicFocusEnhancement") 
		local bonusEnhancementDuration = (enhancementSkill / 100) * 8		
		local bonusHealthPlus = enhancementSkill/100 * self.MaxBonusHealth
		self.Duration = self.Duration + TimeSpan.FromMinutes(bonusEnhancementDuration)	
		-- apply buff icons
		if ( self.Target:IsPlayer() ) then
			AddBuffIcon(self.Target, "Heroism", "Heroism", "Holy Shield", "you feel empowered by the light.")
		end
		SetMobileMod(self.Target, "MaxHealthPlus", "Heroism", self.BaseBonusHealth + bonusHealthPlus)
		self.PlayEffects(self,root,target,args)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.Target:IsPlayer() ) then
			RemoveBuffIcon(self.Target, "Heroism")
			self.Target:SystemMessage("[FFFF77]The light of heroism leaves you...")
		end
		SetMobileMod(self.Target, "MaxHealthPlus", "Heroism", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.Target:PlayObjectSound("event:/magic/misc/magic_water_restoration",false)
		self.Target:PlayEffectWithArgs("HolyEffect",0,"Bone=Ground")
		self.Target:PlayEffectWithArgs("YellowPortalEffect",2.5,"Bone=Ground")
	end,
}