---------------------------------------------------------
-- WorldBuildingWindow
-- Version 1.0
---------------------------------------------------------
MobileEffectLibrary.WorldBuildingWindow = 
{
    -----------------------------------------------------
    -- VARIABLES        
    -----------------------------------------------------
    -- variables
    AddTargetType = nil,
    CurrentWindowState = "Maximized",
    CurrentCategorySelected = "Housing", 
    -- tables
    DynamicWindowData =
    {
        Handle = nil,
        ResponseId = "WorldBuildingWindow",
        Title = "World Building",
    },
    GroupData = 
    {
        Name = "",
        Objects = 
        {
            Buildings = {},
            Decorations = {},
            Lighting = {},
            Spawners = {},
            DungeonTiles={},
        },
    },
    WindowState =
    {
        Maximized = {Anchor = "Center", Height = 600, Width = 240, X = -175, Y = -120},
        Minimized = {Anchor = "Center", Height = 70, Width = 240, X = -112, Y = -35},
    },
    -----------------------------------------------------
    -- CORE FUNCTIONS
    -----------------------------------------------------
    OnEnterState = function(self,root,target,args)
        if(self.ParentObj:HasObjVar("ToolData.WorldBuilding")) then 
            self.GroupData = self.ParentObj:GetObjVar("ToolData.WorldBuilding")
        end       
        self.ShowWorldBuildingWindow(self, root)
        RegisterEventHandler(EventType.DynamicWindowResponse, self.DynamicWindowData.ResponseId,function( user, id )
            self.HandleWorldBuildingWindow( user, id, self, root )
        end)
        RegisterEventHandler(EventType.DynamicWindowResponse, "GroupPickerResponse",function( user, id, textField )
            self.HandleGroupPickerWindow( user, id, textField, self, root )
        end)
        RegisterEventHandler(EventType.ClientTargetGameObjResponse, "AddToGroupDataObjects", function(target,user)
            self.HandleAddToGroupDataObjectsTarget(target,user,self,root)
        end)
    end,
    OnExitState = function(self,root)
        self.ParentObj:CloseDynamicWindow(self.DynamicWindowData.ResponseId)
        UnregisterEventHandler("",EventType.DynamicWindowResponse,self.DynamicWindowData.ResponseId)
        UnregisterEventHandler("",EventType.DynamicWindowResponse,"GroupPickerResponse")
        UnregisterEventHandler("",EventType.ClientTargetGameObjResponse,"AddToGroupDataObjects") 
    end,
    -----------------------------------------------------
    -- CUSTOM FUNCTIONS
    -----------------------------------------------------    
    --- copies the the formatted group element string
    --- to the clip board for pasting into the
    --- mapdata/regionname/seedobjects.xml file
    ClipBoard = function (self,root)
        -- copy to clipboard
        io.popen('clip','w'):write(self.FormatGroupString(self,root)):close()
    end,
    --- format the group xml table from the clipstring table 
    --- and return the output    
    FormatGroupString = function (self,root)
        -- local groupOutput = "<Group>\n\t<DynamicObject><ObjectCreationParams>test data</ObjectCreationParams></DynamicObject>\n</Group>"
        local groupData = self.GroupData
        -- build group element
        local groupElement = xml.new("Group")
        groupElement["Name"] = groupData.Name or "BAD_GROUP_DATA_NAME"
        groupElement["Exclude"] = "False"
        --loop through the self.GroupData.Objects table
        for objectsKey,objectTable in pairs(groupData.Objects) do
            -- loop through each table and grab the dynamicobject element info
            for dynamicObjectKey, dynamicObjectTable in pairs(objectTable) do    
                -- build dynamic object element
                local dynamicObjectElement = xml.new("DynamicObject")
                dynamicObjectElement:append("Name")[1] = objectsKey
                local creationParams = dynamicObjectTable.Template .. " " .. 
                                       dynamicObjectTable.Location.X .. " " ..
                                       dynamicObjectTable.Location.Y .. " " ..
                                       dynamicObjectTable.Location.Z .. " " ..
                                       dynamicObjectTable.Rotation.X .. " " ..
                                       dynamicObjectTable.Rotation.Y .. " " ..
                                       dynamicObjectTable.Rotation.Z .. " " ..
                                       dynamicObjectTable.Scale.X .. " " ..
                                       dynamicObjectTable.Scale.Y .. " " ..
                                       dynamicObjectTable.Scale.Z
                -- add the objectcreationparams element and element data
                dynamicObjectElement:append("ObjectCreationParams")[1] = creationParams
                -- build object variable overrides element
                local objVarOverridesElement = xml.new("ObjVarOverrides")
                local seedGroupVariable = xml.new("StringVariable")
                seedGroupVariable["SeedGroup"] = groupData.Name or "BAD_GROUP_DATA_NAME"
                objVarOverridesElement:append(seedGroupVariable)
                -- add any additional objvars
                -- if(#dynamicObjectTable.ObjVars>0) then
                --     --todo handle objvars
                --     for objVarName, objVarValue in pairs(dynamicObjectTable.ObjVars) do
                --         if(objVarValue ~= nil) then
                --             if(type(objVarValue) == "string") then

                --             elseif(type(objVarValue) == "number") then

                --             elseif(type(objVarValue) == "bool") then

                --             end
                --         end
                --     end
                -- end
                -- add this dynamic object element to the group
                groupElement:append(dynamicObjectElement)
            end
        end
        local clipString = string.gsub(tostring(groupElement),"&quot;","\"")
        clipString = string.sub(clipString,1,-2)
        return clipString
    end,
    HandleGroupPickerWindow = function( user, buttonId, textField, self, root )
        if(buttonId == "SaveGroupName") then
            if(textField.GroupName) then
                self.GroupData.Name = textField.GroupName
                self.ShowWorldBuildingWindow(self,root)
            end
        end
    end,
    HandleWorldBuildingWindow = function( user, buttonId, self, root )
        if(buttonId == nil or buttonId == "" or self.ParentObj ~= user) then 
            EndMobileEffect(root)
            return
        end
        if(buttonId == "MaximizeWindow") then
            self.CurrentWindowState = "Maximized"
            self.ShowWorldBuildingWindow(self,root)
        end
        if(buttonId == "MinimizeWindow") then
            self.CurrentWindowState = "Minimized"
            self.ShowWorldBuildingWindow(self,root)
        end
        if(buttonId=="ClipDataToClipBoard") then
            self.ClipBoard(self,root)
            user:SystemMessage(self.GroupData.Name .. " is waiting on the clip board","event")
        end
        local buttonData = StringSplit(buttonId, "|")
        if(buttonData[1] == "AddSeedObject") then
            self.AddTargetType = buttonData[2]
            user:SystemMessage("select a target to add to the group: " .. self.GroupData.Name,"info")
            self.ParentObj:RequestClientTargetGameObj(self.ParentObj, "AddToGroupDataObjects")           
            self.ShowWorldBuildingWindow(self,root)
        end
    end,
    HandleAddToGroupDataObjectsTarget = function(target,user,self,root)
        if(target) then
            if(self.AddTargetType) then
                local objVars = nil
                if(self.AddTargetType == "Spawners") then
                    -- list of desired simple_mob_spawner objvars to carry over
                    objVars = 
                    {
                        spawnTemplate = nil,
                        spawnRadius = nil,
                        spawnDelay = nil,
                        spawnPulseSecs = nil,
                        spawnChance = nil,
                    }
                    -- grab all objvar data from the targeted simple_mob_spawner
                    for objVarName,v in pairs(objVars) do
                        if(target:HasObjVar(objVarName)) then
                            objVars[objVarName] = target:GetObjVar(objVarName)
                        end
                    end
                    table.insert(self.GroupData.Objects.Spawners,{Template=target:GetCreationTemplateId(),Location = target:GetLoc(),Rotation=target:GetRotation(),Scale=target:GetScale(),ObjVars=objVars})
                    user:SystemMessage("object added to spawners group")
                elseif(self.AddTargetType == "Buildings") then
                    table.insert(self.GroupData.Objects.Buildings,{Template=target:GetCreationTemplateId(),Location = target:GetLoc(),Rotation=target:GetRotation(),Scale=target:GetScale(),ObjVars=objVars})
                    user:SystemMessage("object added to buildings group")
                elseif(self.AddTargetType == "Decorations") then
                    table.insert(self.GroupData.Objects.Decorations,{Template=target:GetCreationTemplateId(),Location = target:GetLoc(),Rotation=target:GetRotation(),Scale=target:GetScale(),ObjVars=objVars})
                    user:SystemMessage("object added to decorations group")
                elseif(self.AddTargetType == "Lighting") then
                    table.insert(self.GroupData.Objects.Lighting,{Template=target:GetCreationTemplateId(),Location = target:GetLoc(),Rotation=target:GetRotation(),Scale=target:GetScale(),ObjVars=objVars})
                    user:SystemMessage("object added to lighting group")
                elseif(self.AddTargetType == "DungeonTiles") then
                    table.insert(self.GroupData.Objects.DungeonTiles,{Template=target:GetCreationTemplateId(),Location = target:GetLoc(),Rotation=target:GetRotation(),Scale=target:GetScale(),ObjVars=objVars})
                    user:SystemMessage("object added to dungeon tiles group")
                end
                self.ShowWorldBuildingWindow(self,root)
            end
        else
            self.ParentObj:SystemMessage("no target selected")
        end
        user:SetObjVar("ToolData.WorldBuilding",self.GroupData)
    end,
    ShowGroupPickerWindow = function(self, root)        
        local dynamicWindow = DynamicWindow("GroupPickerResponse","", 300, 130, -150, -65, "TransparentDraggable", "Center")        
        dynamicWindow:AddImage(0, 0, "Blank", 300 , 130, "Sliced", "000000", 0, 0.8)       
        dynamicWindow:AddImage(-5, -5, "CraftingItemsFrame", 310 , 140, "Sliced", "FF7700", 0, 1)       
        dynamicWindow:AddLabel(150,8,"Enter Group Name", 0, 0, 32, "center", false, false,"SpectralSC-SemiBold")        
        dynamicWindow:AddTextField(8,40,276,20,"GroupName","")
        
        dynamicWindow:AddImage(5, 65, "CraftingItemsFrame", 135 , 26, "Sliced", "FF7700", 0, 1)       
        dynamicWindow:AddLabel(75,74,"Okay", 0, 0, 22, "center", false, false,"SpectralSC-SemiBold")        
        dynamicWindow:AddButton(5,65,"SaveGroupName","",130,26,"[FF7700]Save Group Name[-]\nall seeed objects require a group to be in","",true,"Invisible")        
        

        dynamicWindow:AddImage(155, 65, "CraftingItemsFrame", 135 , 26, "Sliced", "FF7700", 0, 1)       
        dynamicWindow:AddLabel(225,74,"Cancel", 0, 0, 22, "center", false, false,"SpectralSC-SemiBold")                
        dynamicWindow:AddButton(155,65,"CancelGroupNaming","",130,26,"[FF7700]Cancel Group Name[-]\nall seeed objects require a group to be in","",true,"Invisible")        
        
        self.ParentObj:OpenDynamicWindow(dynamicWindow)

    end,
    ShowWorldBuildingWindow = function(self, root)        
        self.DynamicWindowData.Handle = DynamicWindow(self.DynamicWindowData.ResponseId,self.DynamicWindowData.Title, self.WindowState[self.CurrentWindowState].Width, self.WindowState[self.CurrentWindowState].Height, self.WindowState[self.CurrentWindowState].X, self.WindowState[self.CurrentWindowState].Y, "TransparentDraggable", self.WindowState[self.CurrentWindowState].Anchor)        
        self.DynamicWindowData.Handle:AddImage(0, 0, "Blank", self.WindowState[self.CurrentWindowState].Width , self.WindowState[self.CurrentWindowState].Height, "Sliced", "000000", 0, 0.8)       
        self.DynamicWindowData.Handle:AddImage(-5, -5, "CraftingItemsFrame", self.WindowState[self.CurrentWindowState].Width+10 , self.WindowState[self.CurrentWindowState].Height+10, "Sliced", "FF7700", 0, 1)       
        self.DynamicWindowData.Handle:AddLabel(self.WindowState[self.CurrentWindowState].Width/2,8,self.DynamicWindowData.Title, 0, 0, 32, "center", false, false,"SpectralSC-SemiBold")        

        -- MINIMIZED
        if(self.CurrentWindowState == "Minimized") then
            -- Create Buildings
            self.DynamicWindowData.Handle:AddImage(10,self.WindowState[self.CurrentWindowState].Height-35, "achievement_Activity_HouseBuilding",30,30,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(10,self.WindowState[self.CurrentWindowState].Height-35,"AddSeedObject|Buildings","",30,30,"[FF7700]Add Building[-]\nAdd buildings and doors to this group.","",false,"Invisible")
            -- Create Decorations
            self.DynamicWindowData.Handle:AddImage(45,self.WindowState[self.CurrentWindowState].Height-35, "achievement_PvP_VersusRed",30,30,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(45,self.WindowState[self.CurrentWindowState].Height-35,"AddSeedObject|Decorations","",30,30,"[FF7700]Add Decoration[-]\nAdd decorations like tables, keys, crates to this group","",false,"Invisible")            
            -- Create Lighting
            self.DynamicWindowData.Handle:AddImage(80,self.WindowState[self.CurrentWindowState].Height-35, "achievement_Boss Kills_Champion",30,30,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(80,self.WindowState[self.CurrentWindowState].Height-35,"AddSeedObject|Lighting","",30,30,"[FF7700]Add Lighting[-]\nAdd lighting objects to this group","",false,"Invisible")            
            -- Create Spawner
            self.DynamicWindowData.Handle:AddImage(115,self.WindowState[self.CurrentWindowState].Height-35, "achievement_Executioner_Dragon",30,30,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(115,self.WindowState[self.CurrentWindowState].Height-35,"AddSeedObject|Spawners","",30,30,"[FF7700]Add Spawners[-]\nOnly add simple_mob_spawner objects to this group","",false,"Invisible")
            -- Dungeon Tiles
            self.DynamicWindowData.Handle:AddImage(150,self.WindowState[self.CurrentWindowState].Height-35, "achievement_Activity_LandOwner",30,30,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(150,self.WindowState[self.CurrentWindowState].Height-35,"AddSeedObject|DungeonTiles","",30,30,"[FF7700]Add Dungeon Tiles[-]\nOnly add dungeon tile objects to this group","",false,"Invisible")

            -- maximize button
            self.DynamicWindowData.Handle:AddImage(self.WindowState[self.CurrentWindowState].Width-10,-10, "PlusRoundButton_Click",20,20,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(self.WindowState[self.CurrentWindowState].Width-10,-10,"MaximizeWindow","",20,20,"[FF7700]Maximize Window[-]\nMaximizes the window","",false,"Invisible")        
            -- clip it button
            self.DynamicWindowData.Handle:AddImage(-10,-10, "SkillsButton_Click",20,20,"Sliced","FFAA55",0,1)
            self.DynamicWindowData.Handle:AddButton(-10,-10,"ClipDataToClipBoard","",20,20,"[FF7700]Clip Data[-]\nConverts your group to an xml format to paste into your seedobjects.xml","",false,"Invisible")        

        -- MAXIMIZED
        else
            -- if we do not have a group name get one from the user
            if(self.GroupData.Name == "") then
                self.ShowGroupPickerWindow(self,root)
            else
                -- minimize button 
                self.DynamicWindowData.Handle:AddImage(self.WindowState[self.CurrentWindowState].Width-10,-10, "MinusRoundButton_Click",20,20,"Sliced","FFAA55",0,1)            
                self.DynamicWindowData.Handle:AddButton(self.WindowState[self.CurrentWindowState].Width-10,-10,"MinimizeWindow","",20,20,"[FF7700]Minimize Window[-]\nMinimizes the window","",false,"Invisible")

                -- clip it button    
                self.DynamicWindowData.Handle:AddImage(-10,-10, "SkillsButton_Click",20,20,"Sliced","FFAA55",0,1)
                self.DynamicWindowData.Handle:AddButton(-10,-10,"ClipDataToClipBoard","",20,20,"[FF7700]Clip Data[-]\nConverts your group to an xml format to paste into your seedobjects.xml","",false,"Invisible")        

                -- group name
                self.DynamicWindowData.Handle:AddLabel(self.WindowState[self.CurrentWindowState].Width/2,38,"[FF7700]Group Name:[-] "..self.GroupData.Name, 0, 0, 18, "center", false, false,"SpectralSC-SemiBold")

            end
        end

        self.ParentObj:OpenDynamicWindow(self.DynamicWindowData.Handle)
    end,
}
