require 'default:globals.static_data.spells'

-- TODO: RULES FOR SPELL RANGES

---------------------------------------------------------
-- SPELL DURATION RULES
---------------------------------------------------------
-- base effect durations based on spell type
---------------------------------------------------------
SpellData.SpellEffectDuration =
{
	BuffLong = TimeSpan.FromMinutes(5),
	BuffNormal = TimeSpan.FromMinutes(2),
	BuffShort = TimeSpan.FromSeconds(30),
	BuffVeryShort = TimeSpan.FromSeconds(10),
	DebuffLong = TimeSpan.FromMinutes(1),
	DebuffNormal = TimeSpan.FromSeconds(30),
	DebuffShort = TimeSpan.FromSeconds(15),
	Instant = TimeSpan.FromMilliseconds(100),
	Summoning = TimeSpan.FromMinutes(2.5),
	Portal = TimeSpan.FromSeconds(1),
}

---------------------------------------------------------
-- SPELL CAST TIME RULES
---------------------------------------------------------
-- the amount of seconds it takes to cast this spell type
---------------------------------------------------------
SpellData.SpellCastTimes =
{
	BuffLong = 6,
	BuffNormal = 4.5,
	BuffShort = 3,
	BuffInstant = 1,
	DamageHigh = 2.25,
	DamageLow = 1.25,
	DamageNormal = 1.75,
	DebuffLong = 2.5,
	DebuffNormal = 2,
	DebuffShort = 1.25,
	HealHigh = 2.75,
	HealLow = 1.25,
	HealNormal = 2,
	SummoningFast = 1.25,
	SummoningLong = 10,
	SummoningNormal = 5,
	Utility = 2.5,
	Portal = 8,
}

---------------------------------------------------------
-- CASTING SKILL REQUIREMENT RULES
---------------------------------------------------------
-- the min skill to cast and max level to gain skill
---------------------------------------------------------
SpellData.SpellSkillMinMax =
{
	Tier1 = {Min = 0.0,  Max = 20},
	Tier2 = {Min = 15.0, Max = 40},
	Tier3 = {Min = 35.0, Max = 60},
	Tier4 = {Min = 55.0, Max = 80},
	Tier5 = {Min = 75.0, Max = 100},
}

---------------------------------------------------------
-- MANA REQUIREMENT RULES
---------------------------------------------------------
-- the base mana cost for spells
---------------------------------------------------------
SpellData.SpellManaCost =
{
	BuffLong = 22,
	BuffNormal = 14,
	BuffShort = 8,
	DamageHigh = 24,
	DamageLow = 12,
	DamageNormal = 18,
	DebuffLong = 14,
	DebuffNormal = 10,
	DebuffShort = 6,
	HealHigh = 32,
	HealLow = 8,
	HealNormal = 20,
	SummoningHigh = 40,
	SummoningNormal = 20,
	SummoningLow = 8,
	Utility = 16,
	Portal = 50,	
}

---------------------------------------------------------
-- SPELL TEXTCOLOR RULES
---------------------------------------------------------
-- colors for text appearence in game
---------------------------------------------------------

SpellData.TextColors =
{
	-- spell casting power word text color
	PowerWords =
	{
		Dark = "[AA55AA]",
		Light = "[AAAA00]",
		Air = "[4477AA]",
		Fire = "[FF9922]",
	},
}

---------------------------------------------------------
-- BASE SPELL DEFINITIONS
---------------------------------------------------------
-- custom spell definition files by school type
---------------------------------------------------------
require 'globals.static_data.spells.air'
require 'globals.static_data.spells.dark'
require 'globals.static_data.spells.earth'
require 'globals.static_data.spells.fire'
require 'globals.static_data.spells.light'
require 'globals.static_data.spells.water'