---------------------------------------------------------
-- NAMESPACE DEFINITIONS
---------------------------------------------------------
-- the default login_player script as we use some of it
require 'default:login_player'
-- include the tables needed for the login player 
require 'login_player_tables'
-- required overrides of the base login player 
require 'login_player_overrides'

---------------------------------------------------------
-- PUBLIC VARIABLES
---------------------------------------------------------
-- string value holding the table name key in the StartingCharacterBuilds table 
SelectedCharacterBuild = "ClericOfTheLight"
SelectedRace = "Human"
	
---------------------------------------------------------
-- CUSTOM FUNCTIONS
---------------------------------------------------------
--- container function to place the background
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - color (String) a hexidecimal format color value
function DrawBackground(dynamicWindow,color)
	dynamicWindow:AddImage(-220,-400,"HueSwatch",2400,1200,"Simple",color)
end

--- container function to designate the location of the logo text on the screen
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
function DrawLogoText(x,y,dynamicWindow)
	dynamicWindow:AddLabel(x,y,"[" .. ColorDefinitions.TextBackground .. "]Forsaken Realms",800,0,146,"center")
	dynamicWindow:AddLabel(x+2,y+2,"[" .. ColorDefinitions.Text .. "]Forsaken Realms",800,0,144,"center")
end

--- container function to designate the location of the logo image on the screen
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
function DrawLogoImage(x,y,dynamicWindow)
	dynamicWindow:AddImage(x,-180,"season1_2",514,514,"Simple")		
	dynamicWindow:AddImage(x-10,y-10,"FancyDivider",534,20,"Sliced",ColorDefinitions.DividerColor)			
	dynamicWindow:AddImage(x-10,y+308,"FancyDivider",534,20,"Sliced",ColorDefinitions.DividerColor)			
	dynamicWindow:AddLabel(x+257,y+328,"[" .. ColorDefinitions.Text .. "]Welcome to Forsaken Realms",534,0,26,"center")	
	dynamicWindow:AddImage(x-10,y+348,"FancyDivider",534,20,"Sliced",ColorDefinitions.DividerColor)			
end

--- container function to designate the location of the character portrait on the screen
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
-- @param - excludeName (Bool) false to show name, true to exclude it from being displayed
function DrawCharacterPortrait(x,y,dynamicWindow,excludeName)
	if not(excludeName) then
		dynamicWindow:AddLabel(x+160,y-40,"[" .. ColorDefinitions.Text .. "][b]" .. this:GetName() .. "[/b][-]",320,0,22,"center")
	end
	dynamicWindow:AddPortrait(x,y,320,320,this,"Full",true)
end

--- container function to designate the location of the template selection on the screen
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
function DrawTemplateSelection(x,y,dynamicWindow)
	local scrollWindow = ScrollWindow(x,y-30,370,330,30)
	local scrollElement = ScrollElement()
	dynamicWindow:AddLabel(170+x,y-62,"[" .. ColorDefinitions.Text2 .. "][b]Select a starting template[/b][-]",270,0,24,"center")
	for startingBuildsKey,startingBuildsValue in pairs(StartingCharacterBuilds) do
		scrollElement:AddImage(0,0,"HueSwatch",350,30,"Simple",ColorDefinitions.ButtonBorder)
		scrollElement:AddImage(2,2,"HueSwatch",348,26,"Simple",ColorDefinitions.ButtonBackground)
		scrollElement:AddLabel(10,6,"[" .. ColorDefinitions.ButtonText .."]" .. StartingCharacterBuilds[startingBuildsKey].DisplayName,340,0,24,"left")
		scrollElement:AddButton(0,0,"SelectTemplate|"..startingBuildsKey,"",350,30,"[BBAA33]Select the [CCCC00]"..StartingCharacterBuilds[startingBuildsKey].DisplayName,nil,false,"Invisible")
		scrollWindow:Add(scrollElement)					
		scrollElement = ScrollElement()
	end	
	dynamicWindow:AddScrollWindow(scrollWindow)
end

-- container function to designate the location of the introduction text
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
function DrawTemplateIntroduction(x,y,dynamicWindow)
	dynamicWindow:AddImage(x-15,y-100,"FancyDivider",430,20,"Sliced",ColorDefinitions.DividerColor)
	dynamicWindow:AddLabel(x+200,y-80,"[" .. ColorDefinitions.Text .. "]Template Introduction",400,0,25,"center")	
	dynamicWindow:AddImage(x-15,y-60,"FancyDivider",430,20,"Sliced",ColorDefinitions.DividerColor)
	dynamicWindow:AddImage(x-27,y-104,StartingCharacterBuilds[SelectedCharacterBuild].Icon.Primary,64,64,"Sliced",ColorDefinitions.IntroductionIcon)
	dynamicWindow:AddImage(x+373,y-104,StartingCharacterBuilds[SelectedCharacterBuild].Icon.Secondary,64,64,"Sliced",ColorDefinitions.IntroductionIcon)
	dynamicWindow:AddLabel(x,y-20,"[" .. ColorDefinitions.Text .. "]" .. StartingCharacterBuilds[SelectedCharacterBuild].Introduction,400,425,25,"left")	
end

-- container function to designate the location of the create character button
-- @param - dynamiceWindow (DynamicWindow) dynamicw window to draw the background on
-- @param - x (Number) the x location of the container window
-- @param - y (Number) the y location of the container window
function DrawCreateCharacterButton(x,y,dynamicWindow)
	dynamicWindow:AddImage(x,y,"NextButton_Default",420,60,"Sliced","FF7700")
	dynamicWindow:AddLabel(x+210,y+12,"[" .. ColorDefinitions.Text2 .. "]Enter World[-]",420,60,48,"center")
	dynamicWindow:AddButton(x,y,"EnterTheWorld","",420,60,StartingCharacterBuilds[SelectedCharacterBuild].EnterWorldButtonTooltip,nil,false,"Invisible")
end

--- dispaly the character creation menu
function OpenWelcomeWindow()
	local dynamicWindow = DynamicWindow("TemplateSelectionWindow","",320,300,200,-150,"Transparent","Left")
	DrawBackground(dynamicWindow,ColorDefinitions.Black)
	DrawLogoText(764,-308,dynamicWindow)
	DrawLogoImage(510,-180,dynamicWindow)
	DrawCharacterPortrait(390,260,dynamicWindow,false)
	DrawTemplateSelection(655,270,dynamicWindow)
	DrawTemplateIntroduction(0,228,dynamicWindow)
	DrawCreateCharacterButton(1200,530,dynamicWindow)
	this:OpenDynamicWindow(dynamicWindow)	
end

-- container function to designate the location of the create character button
-- @param - user (GameObject) the game object we are working with
-- @param - buttonId (String) the button identifier that called this event
-- @param - textField (Table) a table containing textfield identifiers
function HandleTemplateSelectionWindow(user, buttonId, textField)
	if(buttonId == "EnterTheWorld") then
		curCity = 1
		SetStartingSkills()
		SetRacialStats()
		EnterWorld(curCity)
		return
	end
	-- filter other button options out
	local result = StringSplit(buttonId,"|")
	local button = result[1]
	local args = result[2]
	if(result and button == "SelectTemplate") then
		if(args) then
			SelectedCharacterBuild = args
			ChangeToTemplate(StartingCharacterBuilds[SelectedCharacterBuild].Template,{KeepAppearance = false, LoadAbilities = false, SetStats = true, SetName = false, BuildHotbar = false, LoadLoot=true, LoadEquipment=true})
			GoState("Welcome")
		end
	end
end

-- set the starting stats based on race
function SetRacialStats()
	SetAgi(this, RaceDefinitions[SelectedRace].Stats.Agi,newValue or 10)	
	SetInt(this, RaceDefinitions[SelectedRace].Stats.Int,newValue or 10)	
	SetStr(this, RaceDefinitions[SelectedRace].Stats.Str,newValue or 10)	
	SetWis(this, RaceDefinitions[SelectedRace].Stats.Wis,newValue or 10)	
	SetWill(this, RaceDefinitions[SelectedRace].Stats.Will,newValue or 10)	
end

-- set the starting skills
function SetStartingSkills()
	for k,v in pairs(StartingCharacterBuilds[SelectedCharacterBuild].Skills) do
		SetSkillLevel(this, v.SkillName, v.StartValue, false)
	end
end

RegisterEventHandler(EventType.DynamicWindowResponse, "TemplateSelectionWindow",HandleTemplateSelectionWindow)