-- how often the logic is checked on the tribute spawner
LogicCheckTick = TimeSpan.FromSeconds(10.0)
-- the current tribute table key (check the tribute_spawnertables.lua file)
TributeKey = ""

--- generates the needed information and tables for this tribut spawner
function GenerateTributeSpawnData()
	-- sinle array holding eacy wave being completed, set all to false on startup
	local waveTable = {}
	for wave, waveData in pairs(TributeSpawns[TributeKey].Waves) do
		waveTable[wave] = {Completed = false, Spawned = false}
	end
	this:SetObjVar("WavesTable",waveTable)
	this:SetObjVar("WaveSpawns",{})
end

--- handles the event when a object is created by this tribute spawner
-- @param success (bool) -
-- @param spawnedObject (object) -
function HandleSpawnedObject(success, spawnedObject)
	if(success) then
		local waveSpawns = this:GetObjVar("WaveSpawns")
		table.insert(waveSpawns,spawnedObject)
		this:SetObjVar("WaveSpawns",waveSpawns)
	end
end

--- handles the start tribute spawn message which fires from the tribute_spawner_controller.lua file
-- @param tributeTableKey (table) -
function HandleStartTributeSpawn(tributeTableKey)
	TributeKey = tributeTableKey or ""
	this:SetObjVar("CurrentWave",1)
	GenerateTributeSpawnData()
	StartTributeSpawn()
end

--- when the timer puls event fires this function runs the logic of spawning
function TimerPulse()
	local wavesTable = this:GetObjVar("WavesTable") 
	local currentWave = this:GetObjVar("CurrentWave")
	if (wavesTable == nil or currentWave == nil) then return end
	-- if the wave isn't spawned yet, spawn it
	if not(wavesTable[currentWave].Spawned) then
		SpawnTributeWave()
		StartTributeSpawn()
	-- otherwise check to see if the current wave is completed.
	else
		if (ValidateSpawnedObjects()) then
			StartTributeSpawn()
		end
	end
end

--- when spawn is done destroy the altar
function DestroyTributeAltar()
	local tributeInfo = TributeSpawns[this:GetObjVar("TributeTableKey")].RequiredTributeObjects[TributeKey]
	this:PlayEffect(tributeInfo.EffectWhenPlaced)
	this:PlayEffect("LightningEffect")
	this:PlayObjectSound(tributeInfo.SoundWhenPlaced)
	this:PlayObjectSound("event:/magic/air/magic_air_lightning_impact")
	CallFunctionDelayed(TimeSpan.FromSeconds(5),function ( ... )
        this:Destroy()
    end)
end

---  spawn the current wave
function SpawnTributeWave()
	local currentWave = this:GetObjVar("CurrentWave")
	local minSpawn = TributeSpawns[TributeKey].Waves[currentWave].Quantity.Min
	local maxSpawn = TributeSpawns[TributeKey].Waves[currentWave].Quantity.Max
	local placeHolderChance = TributeSpawns[TributeKey].Waves[currentWave].PlaceHolderChance
	local spawnEntries = TributeSpawns[TributeKey].Waves[currentWave].NormalEntries
	local placeHolderEntries = TributeSpawns[TributeKey].Waves[currentWave].PlaceHolderEntries
	local wavesTable = this:GetObjVar("WavesTable")
	local amountToSpawn = math.random(minSpawn,maxSpawn) or minSpawn
	local template = ""
	for index=1, amountToSpawn do
		template = spawnEntries[math.random(#spawnEntries)]
		if(placeHolderChance and placeHolderEntries) then
			if(math.random() <= placeHolderChance) then 
				template = placeHolderEntries[math.random(#placeHolderEntries)] 
			end
		end
		-- SPAWN OBJECT
		local spawnRadius = math.random(TributeSpawns[TributeKey].SpawnRange.Min,TributeSpawns[TributeKey].SpawnRange.Max)
		local spawnLocation = GetRandomPassableLocationInRadius(this:GetLoc(),spawnRadius,true)
		CreateObj(template, spawnLocation, "TributeSpawnedObject")
	end
	wavesTable[currentWave].Spawned = true
	this:SetObjVar("WavesTable",wavesTable)
end

--- kick off the timer
function StartTributeSpawn()
	this:ScheduleTimerDelay(LogicCheckTick, "TributeSpawnPulse")
end

--- validates the current wave of spawns and sets up the next wave if available
function ValidateSpawnedObjects()
	local waveSpawnsTable = this:GetObjVar("WaveSpawns")
	local count = 0
	for k,v in pairs(waveSpawnsTable) do
		if(v:IsMobile() and IsDead(v) or not v:IsValid()) then count = count + 1 end
	end
	if(count >= #waveSpawnsTable) then
		local wavesTable = this:GetObjVar("WavesTable")
		local currentWave = this:GetObjVar("CurrentWave")
		wavesTable[currentWave].Completed = true
		this:SetObjVar("WavesTable",wavesTable)
		this:SetObjVar("WaveSpawns",{})
		currentWave = currentWave + 1
		this:SetObjVar("CurrentWave",currentWave)
		if(currentWave > #wavesTable) then
			this:Destroy()
			return false
		end
	end

	return true
end

--- register the event handler for when the timer event fires on the tribute spawner
RegisterEventHandler(EventType.Timer, "TributeSpawnPulse", TimerPulse)

--- register the event handler for when a object is spawned by the tribute spawner
RegisterEventHandler(EventType.CreatedObject,"TributeSpawnedObject",HandleSpawnedObject)