require 'tribute_spawner'

--- bulids a container table to store the tribute key information
-- @param tributeSpawnsKey (string) - table key for this tribute spawners logic and information
function BuildRequiredTributeTables(tributeSpawnsKey)
	local tributeTable = {}
	if(TributeSpawns[tributeSpawnsKey]) then
		if(TributeSpawns[tributeSpawnsKey].RequiredTributeObjects) then
			for key,value in pairs(TributeSpawns[tributeSpawnsKey].RequiredTributeObjects) do
				table.insert(tributeTable,{key,false})
			end
		end
	end
	this:SetObjVar("TributeTable",tributeTable)
end

--- check to see if all the tributes have been paid
-- @param user (object) - player who is interacting with the tribute spawner
function CheckTribute(user)
	local tributeTable = this:GetObjVar("TributeTable")
	local tributesComplete = 0
	local tributeTableKey = this:GetObjVar("TributeTableKey")
	for key,value in pairs(tributeTable) do
		if(value[2] == true) then tributesComplete = tributesComplete + 1 end
	end

	if(tributesComplete >= #tributeTable) then
		this:SetObjVar("AllTributePaid",true)
		this:SendMessage("StartTributeSpawn",tributeTableKey)
	end
end

--- handles the event when a player tries give a tribute item
-- @param user (object) - player who is interacting with the tribute spawner
-- @param droppedItem (object) - item currently on the users cursor
function HandleDropedItemRequest(user,droppedItem)
	if(droppedItem) then
		TryToAddTribute(user,droppedItem)
		if(this:GetObjVar("AllTributePaid") == true) then
			user:CloseDynamicWindow("TributeSpawnerWindow")
		end
	end
end

---	handles tribute setup when the module is attached
function HandleModuleAttached()
	if not(this:HasObjVar("TributeTable")) then
		BuildRequiredTributeTables(this:GetObjVar("TributeTableKey"))
	end
	this:SetObjVar("AllTributePaid",false)
end

--- play the tribute effects when items are added
-- @param reqTributeKey (string) - the key for the RequiredTributeObjects table
function PlayTributeEffects(reqTributeKey)
	local tributeInfo = TributeSpawns[this:GetObjVar("TributeTableKey")].RequiredTributeObjects[reqTributeKey]
	this:PlayEffect(tributeInfo.EffectWhenPlaced)
	this:PlayEffect("LightningEffect")
	this:PlayObjectSound(tributeInfo.SoundWhenPlaced)
	this:PlayObjectSound("event:/magic/air/magic_air_lightning_impact")
end

--- checks to see if the droppedItem is needed if so add it to the tribute table
-- @param user (object) - player who is interacting with the tribute spawner
-- @param droppedItem (object) - item currently on the users cursor
function TryToAddTribute(user,droppedItem)
	local tributeTable = this:GetObjVar("TributeTable")
	if(tributeTable) then
		for key,value in pairs(tributeTable) do
			local tributeTemplateNeeded = value[1]
			local tributeAccepted = value[2]
			local droppedTemplate = tostring(droppedItem:GetCreationTemplateId())
			if(tributeTemplateNeeded == droppedTemplate) then
				if (tributeAccepted == false) then
					tributeTable[key][2] = true
					droppedItem:Destroy()
					PlayTributeEffects(droppedTemplate)
				else
					user:SystemMessage("this tribute has already been accepted!")
				end
			else
				user:SystemMessage("this does not seem to fit anywhere that you can tell")
			end
		end
		this:SetObjVar("TributeTable",tributeTable)
		CheckTribute(user)
	end
end

-- register a single event handler for when this module gets attached
RegisterSingleEventHandler(EventType.ModuleAttached, GetCurrentModule(),HandleModuleAttached)

-- register a event handler for when this object recieves the "DroppedItem" message
RegisterEventHandler(EventType.Message, "DroppedItem", HandleDropedItemRequest)

RegisterEventHandler(EventType.Message,"StartTributeSpawn",HandleStartTributeSpawn)